package com.loafle.overflow.central.module.infra.service;

import com.loafle.overflow.central.commons.utils.PageUtil;
import com.loafle.overflow.central.module.infra.dao.InfraDAO;
import com.loafle.overflow.central.module.infra.dao.InfraHostDAO;
import com.loafle.overflow.central.module.infra.dao.InfraHostIPDAO;
import com.loafle.overflow.central.module.infra.dao.InfraServiceDAO;
import com.loafle.overflow.central.module.infra.dao.InfraZoneDAO;
import com.loafle.overflow.central.module.meta.dao.MetaIPTypeDAO;
import com.loafle.overflow.central.module.meta.service.CentralMetaIPTypeService;
import com.loafle.overflow.core.model.PageParams;
import com.loafle.overflow.model.probe.Probe;
import com.loafle.overflow.core.exception.OverflowException;
import com.loafle.overflow.model.discovery.Host;
import com.loafle.overflow.model.discovery.Zone;
import com.loafle.overflow.model.infra.Infra;
import com.loafle.overflow.model.infra.InfraHost;
import com.loafle.overflow.model.infra.InfraHostIP;
import com.loafle.overflow.model.infra.InfraZone;
import com.loafle.overflow.model.meta.MetaIPType;
import com.loafle.overflow.model.meta.MetaInfraType;
import com.loafle.overflow.model.meta.MetaTargetHostType;
import com.loafle.overflow.model.meta.MetaTargetZoneType;
import com.loafle.overflow.model.noauthprobe.NoAuthProbeDescription;
import com.loafle.overflow.model.noauthprobe.NoAuthProbeDescriptionHost;
import com.loafle.overflow.model.noauthprobe.NoAuthProbeDescriptionNetwork;
import com.loafle.overflow.service.central.infra.InfraHostIPService;
import com.loafle.overflow.service.central.infra.InfraService;
import com.loafle.overflow.service.central.meta.MetaIPTypeService;
import com.loafle.overflow.service.central.probe.ProbeService;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

@Service("InfraService")
public class CentralInfraService implements InfraService {
  @Autowired
  ProbeService probeService;

  @Autowired
  InfraDAO infraDAO;

  @Autowired
  InfraZoneDAO infraZoneDAO;

  @Autowired
  InfraHostDAO infraHostDAO;

  @Autowired
  InfraServiceDAO infraServiceDAO;

  @Autowired
  CentralInfraHostIPService infraHostIPService;

  @Autowired
  CentralMetaIPTypeService metaIPTypeService;

  @Override
  public Infra regist(Infra infra) throws OverflowException {
    return this.infraDAO.save(infra);
  }

  @Override
  public InfraZone registZone(Long probeID, Zone zone) throws OverflowException {
    if (null == zone) {
      throw new OverflowException("Zone is not valid");
    }

    Probe probe = this.probeService.read(probeID);
    if (null == probe) {
      throw new OverflowException(String.format("ID[%d] of Probe is not valid", probeID));
    }

    if (null == zone.getNetwork()) {
      throw new OverflowException("Network is not valid");
    }

    if (null == zone.getAddress()) {
      throw new OverflowException("IP is not valid");
    }

    if (null == zone.getMac()) {
      throw new OverflowException("Mac is not valid");
    }

    if (null == zone.getIface()) {
      throw new OverflowException("Iface is not valid");
    }

    if (null == zone.getMetaIPTypeKey()) {
      throw new OverflowException("MetaIPTypeKey is not valid");
    }

    MetaIPType metaIPType = this.metaIPTypeService.readByKey(zone.getMetaIPTypeKey());
    if (null == metaIPType) {
      throw new OverflowException(String.format("MetaIPTypeKey[%s] is not valid", zone.getMetaIPTypeKey()));
    }

    InfraZone infraZone = this.infraZoneDAO.findByProbeIdAndNetwork(probeID, zone.getNetwork());
    if (null != infraZone) {
      return infraZone;
    }

    infraZone = new InfraZone();
    infraZone.setMetaInfraType(MetaInfraType.Enum.ZONE.to());
    infraZone.setProbe(probe);
    infraZone.setMetaTargetZoneType(MetaTargetZoneType.Enum.ZONE.to());
    infraZone.setNetwork(zone.getNetwork());
    infraZone.setIface(zone.getIface());
    infraZone.setMetaIPType(metaIPType);
    infraZone.setAddress(zone.getAddress());
    infraZone.setMac(zone.getMac());

    return this.infraZoneDAO.save(infraZone);
  }

  @Override
  public InfraHost registHost(Long probeID, Host host) throws OverflowException {
    if (null == host) {
      throw new OverflowException("Host is not valid");
    }

    Probe probe = this.probeService.read(probeID);
    if (null == probe) {
      throw new OverflowException(String.format("ID[%d] of Probe is not valid", probeID));
    }

    Zone zone = host.getZone();
    if (null == zone) {
      throw new OverflowException("Zone is not valid");
    }

    InfraZone infraZone = this.registZone(probeID, zone);

    if (null == host.getMetaIPTypeKey()) {
      throw new OverflowException("MetaIPTypeKey is not valid");
    }

    MetaIPType metaIPType = this.metaIPTypeService.readByKey(host.getMetaIPTypeKey());
    if (null == metaIPType) {
      throw new OverflowException(String.format("MetaIPTypeKey[%s] is not valid", host.getMetaIPTypeKey()));
    }

    if (null == host.getAddress()) {
      throw new OverflowException("IP is not valid");
    }

    if (null == host.getMac()) {
      throw new OverflowException("Mac is not valid");
    }

    InfraHostIP infraHostIP = this.infraHostIPService.readByInfraHostInfraZoneIDAndMetaIPTypeKeyAndAddress(infraZone.getId(), metaIPType.getKey(), host.getAddress());
    if (null != infraHostIP) {
      throw new OverflowException(String.format("IP[%s(%s)] of Zone[%s] is exist already", host.getAddress(), metaIPType.getName(), infraZone.getNetwork()));
    }

    infraHostIP = new InfraHostIP();
    infraHostIP.setMetaIPType(metaIPType);
    infraHostIP.setAddress(host.getAddress());
    infraHostIP.setMac(host.getMac());

    InfraHost infraHost = new InfraHost();
    infraZone.setMetaInfraType(MetaInfraType.Enum.HOST.to());
    infraZone.setProbe(probe);
    infraHost.setInfraZone(infraZone);
    infraHost.setMetaTargetHostType(MetaTargetHostType.Enum.UNKNOWN.to());
    infraHost.setInfraHostIPs(Arrays.asList(infraHostIP));

    return this.infraHostDAO.save(infraHost);
  }

  @Override
  public InfraService registService(Long probeID, com.loafle.overflow.model.discovery.Service service)
      throws OverflowException {
    return null;
  }

  @Override
  @Transactional
  public List<Infra> registDiscoverd(Long probeID, List<Host> hosts,
      List<com.loafle.overflow.model.discovery.Service> services) throws OverflowException {
    return null;
  }

  public InfraHost registByNoAuthProbe(Long probeID, NoAuthProbeDescription noAuthProbeDescription)
      throws OverflowException {
    NoAuthProbeDescriptionNetwork networkInfo = noAuthProbeDescription.getNetwork();
    InfraHostIP infraHostIP = new InfraHostIP();
    // FIXME getting IPType from networkInfo
    infraHostIP.setMetaIPType(new MetaIPType((short) 1));
    infraHostIP.setAddress(networkInfo.getAddress());
    infraHostIP.setMac(networkInfo.getMacAddress());
    List<InfraHostIP> infraHostIPs = new ArrayList<>();
    infraHostIPs.add(infraHostIP);

    InfraHost infraHost = new InfraHost();
    infraHost.setProbe(new Probe(probeID));
    infraHost.setMetaInfraType(new MetaInfraType(1));
    infraHost.setMetaTargetHostType(new MetaTargetHostType(1));
    infraHost.setInfraHostIPs(infraHostIPs);

    return this.infraDAO.save(infraHost);
  }

  @Override
  public Infra read(Long id) throws OverflowException {
    return this.infraDAO.findById(id).get();
  }

  @Override
  public Page<Infra> readAllByProbeID(Long probeID, PageParams pageParams) throws OverflowException {
    return this.infraDAO.findAllByProbeId(probeID, PageUtil.getPageRequest(pageParams));
  }

  @Override
  public Page<Infra> readAllByDomainID(Long domainID, PageParams pageParams) throws OverflowException {
    return this.infraDAO.findAllByProbeDomainId(domainID, PageUtil.getPageRequest(pageParams));
  }

}
